package sun.net.www.http;

import java.security.AccessController;
import sun.net.NetProperties;
import java.security.PrivilegedAction;
import java.io.IOException;
import java.util.LinkedList;

class KeepAliveStreamCleaner extends LinkedList<KeepAliveCleanerEntry> implements Runnable
{
    protected static int MAX_DATA_REMAINING;
    protected static int MAX_CAPACITY;
    protected static final int TIMEOUT = 5000;
    private static final int MAX_RETRIES = 5;
    
    KeepAliveStreamCleaner() {
    }
    
    @Override
    public boolean offer(KeepAliveCleanerEntry e) {
        return this.size() < KeepAliveStreamCleaner.MAX_CAPACITY && super.offer(e);
    }
    
    @Override
    public void run() {
        KeepAliveCleanerEntry keepAliveCleanerEntry = null;
        while (true) {
            try {
                synchronized (this) {
                    long currentTimeMillis = System.currentTimeMillis();
                    long timeoutMillis = 5000L;
                    while ((keepAliveCleanerEntry = (KeepAliveCleanerEntry)this.poll()) == null) {
                        this.wait(timeoutMillis);
                        long currentTimeMillis2 = System.currentTimeMillis();
                        long n = currentTimeMillis2 - currentTimeMillis;
                        if (n > timeoutMillis) {
                            keepAliveCleanerEntry = (KeepAliveCleanerEntry)this.poll();
                            break;
                        }
                        currentTimeMillis = currentTimeMillis2;
                        timeoutMillis -= n;
                    }
                }
                if (keepAliveCleanerEntry == null) {
                    break;
                }
                KeepAliveStream keepAliveStream = keepAliveCleanerEntry.getKeepAliveStream();
                if (keepAliveStream != null) {
                    synchronized (keepAliveStream) {
                        HttpClient httpClient = keepAliveCleanerEntry.getHttpClient();
                        try {
                            if (httpClient != null && !httpClient.isInKeepAliveCache()) {
                                int readTimeout = httpClient.getReadTimeout();
                                httpClient.setReadTimeout(5000);
                                long remainingToRead = keepAliveStream.remainingToRead();
                                if (remainingToRead > 0L) {
                                    long skip = 0L;
                                    int n2 = 0;
                                    while (skip < remainingToRead && n2 < 5) {
                                        remainingToRead -= skip;
                                        skip = keepAliveStream.skip(remainingToRead);
                                        if (skip != 0L) {
                                            continue;
                                        }
                                        ++n2;
                                    }
                                    remainingToRead -= skip;
                                }
                                if (remainingToRead == 0L) {
                                    httpClient.setReadTimeout(readTimeout);
                                    httpClient.finished();
                                }
                                else {
                                    httpClient.closeServer();
                                }
                            }
                        }
                        catch (IOException ex) {
                            httpClient.closeServer();
                        }
                        finally {
                            keepAliveStream.setClosed();
                        }
                    }
                }
            }
            catch (InterruptedException ex2) {}
            if (keepAliveCleanerEntry == null) {
                break;
            }
            continue;
        }
    }
    
    static {
        KeepAliveStreamCleaner.MAX_DATA_REMAINING = 512;
        KeepAliveStreamCleaner.MAX_CAPACITY = 10;
        KeepAliveStreamCleaner.MAX_DATA_REMAINING = (int)AccessController.doPrivileged(new PrivilegedAction<Integer>() {
            @Override
            public Integer run() {
                return NetProperties.getInteger("http.KeepAlive.remainingData", KeepAliveStreamCleaner.MAX_DATA_REMAINING);
            }
        }) * 1024;
        KeepAliveStreamCleaner.MAX_CAPACITY = (int)AccessController.doPrivileged(new PrivilegedAction<Integer>() {
            @Override
            public Integer run() {
                return NetProperties.getInteger("http.KeepAlive.queuedConnections", KeepAliveStreamCleaner.MAX_CAPACITY);
            }
        });
    }
}
